# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qm/pb/errors.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='qm/pb/errors.proto',
  package='',
  syntax='proto3',
  serialized_options=b'\n\016qm.grpc.errorsB\013ErrorsProtoP\001\242\002\003HLW',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x12qm/pb/errors.proto*\x89\x02\n\x14JobManagerErrorTypes\x12\x1e\n\x1aJobManagerUnspecifiedError\x10\x00\x12\x14\n\x0fMissingJobError\x10\xe8\x07\x12#\n\x1eInvalidJobExecutionStatusError\x10\xe9\x07\x12(\n#InvalidOperationOnSimulatorJobError\x10\xea\x07\x12#\n\x1eInvalidOperationOnRealJobError\x10\xeb\x07\x12\x1e\n\x19JobOperationSpecificError\x10\xec\x07\x12\x15\n\x10\x43onfigQueryError\x10\xed\x07\"\x05\x08\x01\x10\xe7\x07\"\t\x08\xd0\x0f\x10\xff\xff\xff\xff\x07*\xb8\x02\n\x1eJobOperationSpecificErrorTypes\x12 \n\x1cJobOperationUnspecifiedError\x10\x00\x12\x1c\n\x17SingleInputElementError\x10\xb8\x17\x12!\n\x1cInvalidCorrectionMatrixError\x10\xb9\x17\x12-\n(ElementWithoutIntermediateFrequencyError\x10\xba\x17\x12&\n!InvalidDigitalInputThresholdError\x10\xbb\x17\x12#\n\x1eInvalidDigitalInputWindowError\x10\xbc\x17\x12%\n InvalidDigitalInputPolarityError\x10\xbd\x17\"\x05\x08\x01\x10\xb7\x17\"\t\x08\xa0\x1f\x10\xff\xff\xff\xff\x07*\xa0\x01\n\x15\x43onfigQueryErrorTypes\x12\x1f\n\x1b\x43onfigQueryUnspecifiedError\x10\x00\x12\x1b\n\x16MissingControllerError\x10\xa0\x1f\x12\x18\n\x13MissingElementError\x10\xa1\x1f\x12\x1d\n\x18MissingDigitalInputError\x10\xa2\x1f\"\x05\x08\x01\x10\x9f\x1f\"\t\x08\x88\'\x10\xff\xff\xff\xff\x07\x42%\n\x0eqm.grpc.errorsB\x0b\x45rrorsProtoP\x01\xa2\x02\x03HLWb\x06proto3'
)

_JOBMANAGERERRORTYPES = _descriptor.EnumDescriptor(
  name='JobManagerErrorTypes',
  full_name='JobManagerErrorTypes',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='JobManagerUnspecifiedError', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MissingJobError', index=1, number=1000,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='InvalidJobExecutionStatusError', index=2, number=1001,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='InvalidOperationOnSimulatorJobError', index=3, number=1002,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='InvalidOperationOnRealJobError', index=4, number=1003,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='JobOperationSpecificError', index=5, number=1004,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ConfigQueryError', index=6, number=1005,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=23,
  serialized_end=288,
)
_sym_db.RegisterEnumDescriptor(_JOBMANAGERERRORTYPES)

JobManagerErrorTypes = enum_type_wrapper.EnumTypeWrapper(_JOBMANAGERERRORTYPES)
_JOBOPERATIONSPECIFICERRORTYPES = _descriptor.EnumDescriptor(
  name='JobOperationSpecificErrorTypes',
  full_name='JobOperationSpecificErrorTypes',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='JobOperationUnspecifiedError', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SingleInputElementError', index=1, number=3000,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='InvalidCorrectionMatrixError', index=2, number=3001,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ElementWithoutIntermediateFrequencyError', index=3, number=3002,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='InvalidDigitalInputThresholdError', index=4, number=3003,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='InvalidDigitalInputWindowError', index=5, number=3004,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='InvalidDigitalInputPolarityError', index=6, number=3005,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=291,
  serialized_end=603,
)
_sym_db.RegisterEnumDescriptor(_JOBOPERATIONSPECIFICERRORTYPES)

JobOperationSpecificErrorTypes = enum_type_wrapper.EnumTypeWrapper(_JOBOPERATIONSPECIFICERRORTYPES)
_CONFIGQUERYERRORTYPES = _descriptor.EnumDescriptor(
  name='ConfigQueryErrorTypes',
  full_name='ConfigQueryErrorTypes',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ConfigQueryUnspecifiedError', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MissingControllerError', index=1, number=4000,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MissingElementError', index=2, number=4001,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MissingDigitalInputError', index=3, number=4002,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=606,
  serialized_end=766,
)
_sym_db.RegisterEnumDescriptor(_CONFIGQUERYERRORTYPES)

ConfigQueryErrorTypes = enum_type_wrapper.EnumTypeWrapper(_CONFIGQUERYERRORTYPES)
JobManagerUnspecifiedError = 0
MissingJobError = 1000
InvalidJobExecutionStatusError = 1001
InvalidOperationOnSimulatorJobError = 1002
InvalidOperationOnRealJobError = 1003
JobOperationSpecificError = 1004
ConfigQueryError = 1005
JobOperationUnspecifiedError = 0
SingleInputElementError = 3000
InvalidCorrectionMatrixError = 3001
ElementWithoutIntermediateFrequencyError = 3002
InvalidDigitalInputThresholdError = 3003
InvalidDigitalInputWindowError = 3004
InvalidDigitalInputPolarityError = 3005
ConfigQueryUnspecifiedError = 0
MissingControllerError = 4000
MissingElementError = 4001
MissingDigitalInputError = 4002


DESCRIPTOR.enum_types_by_name['JobManagerErrorTypes'] = _JOBMANAGERERRORTYPES
DESCRIPTOR.enum_types_by_name['JobOperationSpecificErrorTypes'] = _JOBOPERATIONSPECIFICERRORTYPES
DESCRIPTOR.enum_types_by_name['ConfigQueryErrorTypes'] = _CONFIGQUERYERRORTYPES
_sym_db.RegisterFileDescriptor(DESCRIPTOR)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
